@@ -1,10 +1,9 @@
-from __future__ import division
 import numpy as np
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 
 
-__all__ = ["palplot"]
+__all__ = ["palplot", "dogplot"]
 
 
 def palplot(pal, size=1):
@@ -27,3 +26,21 @@ def palplot(pal, size=1):
     ax.set_yticks([-.5, .5])
     ax.set_xticklabels([])
     ax.set_yticklabels([])
+
+
+def dogplot(*_, **__):
+    """Who's a good boy?"""
+    try:
+        from urllib.request import urlopen
+    except ImportError:
+        from urllib2 import urlopen
+    from io import BytesIO
+
+    url = "https://github.com/mwaskom/seaborn-data/raw/master/png/img{}.png"
+    pic = np.random.randint(2, 7)
+    data = BytesIO(urlopen(url.format(pic)).read())
+    img = plt.imread(data)
+    f, ax = plt.subplots(figsize=(5, 5), dpi=100)
+    f.subplots_adjust(0, 0, 1, 1)
+    ax.imshow(img)
+    ax.set_axis_off()
