@@ -1,4 +1,3 @@
-from __future__ import division
 import colorsys
 from itertools import cycle
 
@@ -6,12 +5,9 @@ import numpy as np
 import matplotlib as mpl
 
 from .external import husl
-from .external.six import string_types
-from .external.six.moves import range
 
 from .utils import desaturate, set_hls_values, get_color_cycle
-from .xkcd_rgb import xkcd_rgb
-from .crayons import crayons
+from .colors import xkcd_rgb, crayons
 
 
 __all__ = ["color_palette", "hls_palette", "husl_palette", "mpl_palette",
@@ -21,19 +17,44 @@ __all__ = ["color_palette", "hls_palette", "husl_palette", "mpl_palette",
 
 
 SEABORN_PALETTES = dict(
-    deep=["#4C72B0", "#55A868", "#C44E52",
-          "#8172B2", "#CCB974", "#64B5CD"],
-    muted=["#4878CF", "#6ACC65", "#D65F5F",
-           "#B47CC7", "#C4AD66", "#77BEDB"],
-    pastel=["#92C6FF", "#97F0AA", "#FF9F9A",
-            "#D0BBFF", "#FFFEA3", "#B0E0E6"],
-    bright=["#003FFF", "#03ED3A", "#E8000B",
-            "#8A2BE2", "#FFC400", "#00D7FF"],
-    dark=["#001C7F", "#017517", "#8C0900",
-          "#7600A1", "#B8860B", "#006374"],
-    colorblind=["#0072B2", "#009E73", "#D55E00",
-                "#CC79A7", "#F0E442", "#56B4E9"]
-    )
+    deep=["#4C72B0", "#DD8452", "#55A868", "#C44E52", "#8172B3",
+          "#937860", "#DA8BC3", "#8C8C8C", "#CCB974", "#64B5CD"],
+    deep6=["#4C72B0", "#55A868", "#C44E52",
+           "#8172B3", "#CCB974", "#64B5CD"],
+    muted=["#4878D0", "#EE854A", "#6ACC64", "#D65F5F", "#956CB4",
+           "#8C613C", "#DC7EC0", "#797979", "#D5BB67", "#82C6E2"],
+    muted6=["#4878D0", "#6ACC64", "#D65F5F",
+            "#956CB4", "#D5BB67", "#82C6E2"],
+    pastel=["#A1C9F4", "#FFB482", "#8DE5A1", "#FF9F9B", "#D0BBFF",
+            "#DEBB9B", "#FAB0E4", "#CFCFCF", "#FFFEA3", "#B9F2F0"],
+    pastel6=["#A1C9F4", "#8DE5A1", "#FF9F9B",
+             "#D0BBFF", "#FFFEA3", "#B9F2F0"],
+    bright=["#023EFF", "#FF7C00", "#1AC938", "#E8000B", "#8B2BE2",
+            "#9F4800", "#F14CC1", "#A3A3A3", "#FFC400", "#00D7FF"],
+    bright6=["#023EFF", "#1AC938", "#E8000B",
+             "#8B2BE2", "#FFC400", "#00D7FF"],
+    dark=["#001C7F", "#B1400D", "#12711C", "#8C0800", "#591E71",
+          "#592F0D", "#A23582", "#3C3C3C", "#B8850A", "#006374"],
+    dark6=["#001C7F", "#12711C", "#8C0800",
+           "#591E71", "#B8850A", "#006374"],
+    colorblind=["#0173B2", "#DE8F05", "#029E73", "#D55E00", "#CC78BC",
+                "#CA9161", "#FBAFE4", "#949494", "#ECE133", "#56B4E9"],
+    colorblind6=["#0173B2", "#029E73", "#D55E00",
+                 "#CC78BC", "#ECE133", "#56B4E9"]
+)
+
+
+MPL_QUAL_PALS = {
+    "tab10": 10, "tab20": 20, "tab20b": 20, "tab20c": 20,
+    "Set1": 9, "Set2": 8, "Set3": 12,
+    "Accent": 8, "Paired": 12,
+    "Pastel1": 9, "Pastel2": 8, "Dark2": 8,
+}
+
+
+QUAL_PALETTE_SIZES = MPL_QUAL_PALS.copy()
+QUAL_PALETTE_SIZES.update({k: len(v) for k, v in SEABORN_PALETTES.items()})
+QUAL_PALETTES = list(QUAL_PALETTE_SIZES.keys())
 
 
 class _ColorPalette(list):
@@ -63,19 +84,22 @@ def color_palette(palette=None, n_colors=None, desat=None):
         deep, muted, bright, pastel, dark, colorblind
 
     Other options:
-        hls, husl, any named matplotlib palette, list of colors
+        name of matplotlib cmap, 'ch:<cubehelix arguments>', 'hls', 'husl',
+        or a list of colors in any format matplotlib accepts
 
     Calling this function with ``palette=None`` will return the current
     matplotlib color cycle.
 
     Matplotlib palettes can be specified as reversed palettes by appending
-    "_r" to the name or as dark palettes by appending "_d" to the name.
+    "_r" to the name or as "dark" palettes by appending "_d" to the name.
     (These options are mutually exclusive, but the resulting list of colors
     can also be reversed).
 
     This function can also be used in a ``with`` statement to temporarily
     set the color cycle for a plot or set of plots.
 
+    See the :ref:`tutorial <palette_tutorial>` for more information.
+
     Parameters
     ----------
     palette: None, string, or sequence, optional
@@ -106,40 +130,54 @@ def color_palette(palette=None, n_colors=None, desat=None):
     Examples
     --------
 
-    Show one of the "seaborn palettes", which have the same basic order of hues
-    as the default matplotlib color cycle but more attractive colors.
+    Calling with no arguments returns all colors from the current default
+    color cycle:
 
     .. plot::
         :context: close-figs
 
         >>> import seaborn as sns; sns.set()
+        >>> sns.palplot(sns.color_palette())
+
+    Show one of the other "seaborn palettes", which have the same basic order
+    of hues as the default matplotlib color cycle but more attractive colors.
+    Calling with the name of a palette will return 6 colors by default:
+
+    .. plot::
+        :context: close-figs
+
         >>> sns.palplot(sns.color_palette("muted"))
 
-    Use discrete values from one of the built-in matplotlib colormaps.
+    Use discrete values from one of the built-in matplotlib colormaps:
 
     .. plot::
         :context: close-figs
 
         >>> sns.palplot(sns.color_palette("RdBu", n_colors=7))
 
-    Make a "dark" matplotlib sequential palette variant. (This can be good
-    when coloring multiple lines or points that correspond to an ordered
-    variable, where you don't want the lightest lines to be invisible).
+    Make a customized cubehelix color palette:
 
     .. plot::
         :context: close-figs
 
-        >>> sns.palplot(sns.color_palette("Blues_d"))
+        >>> sns.palplot(sns.color_palette("ch:2.5,-.2,dark=.3"))
 
-    Use a categorical matplotlib palette, add some desaturation. (This can be
-    good when making plots with large patches, which look best with dimmer
-    colors).
+    Use a categorical matplotlib palette and add some desaturation:
 
     .. plot::
         :context: close-figs
 
         >>> sns.palplot(sns.color_palette("Set1", n_colors=8, desat=.5))
 
+    Make a "dark" matplotlib sequential palette variant. (This can be good
+    when coloring multiple lines or points that correspond to an ordered
+    variable, where you don't want the lightest lines to be invisible):
+
+    .. plot::
+        :context: close-figs
+
+        >>> sns.palplot(sns.color_palette("Blues_d"))
+
     Use as a context manager:
 
     .. plot::
@@ -155,25 +193,40 @@ def color_palette(palette=None, n_colors=None, desat=None):
         if n_colors is None:
             n_colors = len(palette)
 
-    elif not isinstance(palette, string_types):
+    elif not isinstance(palette, str):
         palette = palette
         if n_colors is None:
             n_colors = len(palette)
     else:
 
         if n_colors is None:
-            n_colors = 6
+            # Use all colors in a qualitative palette or 6 of another kind
+            n_colors = QUAL_PALETTE_SIZES.get(palette, 6)
 
-        if palette == "hls":
-            palette = hls_palette(n_colors)
-        elif palette == "husl":
-            palette = husl_palette(n_colors)
-        elif palette.lower() == "jet":
-            raise ValueError("No.")
-        elif palette in SEABORN_PALETTES:
+        if palette in SEABORN_PALETTES:
+            # Named "seaborn variant" of old matplotlib default palette
             palette = SEABORN_PALETTES[palette]
+
+        elif palette == "hls":
+            # Evenly spaced colors in cylindrical RGB space
+            palette = hls_palette(n_colors)
+
+        elif palette == "husl":
+            # Evenly spaced colors in cylindrical Lab space
+            palette = husl_palette(n_colors)
+
+        elif palette.lower() == "jet":
+            # Paternalism
+            raise ValueError("No.")
+
+        elif palette.startswith("ch:"):
+            # Cubehelix palette with params specified in string
+            args, kwargs = _parse_cubehelix_args(palette)
+            palette = cubehelix_palette(n_colors, *args, **kwargs)
+
         else:
             try:
+                # Perhaps a named matplotlib colormap?
                 palette = mpl_palette(palette, n_colors)
             except ValueError:
                 raise ValueError("%s is not a valid palette name" % palette)
@@ -195,7 +248,7 @@ def color_palette(palette=None, n_colors=None, desat=None):
     return palette
 
 
-def hls_palette(n_colors=6, h=.01, l=.6, s=.65):
+def hls_palette(n_colors=6, h=.01, l=.6, s=.65):  # noqa
     """Get a set of evenly spaced colors in HLS hue space.
 
     h, l, and s should be between 0 and 1
@@ -255,7 +308,7 @@ def hls_palette(n_colors=6, h=.01, l=.6, s=.65):
         >>> sns.palplot(sns.hls_palette(10, s=.4))
 
     """
-    hues = np.linspace(0, 1, n_colors + 1)[:-1]
+    hues = np.linspace(0, 1, int(n_colors) + 1)[:-1]
     hues += h
     hues %= 1
     hues -= hues.astype(int)
@@ -263,7 +316,7 @@ def hls_palette(n_colors=6, h=.01, l=.6, s=.65):
     return _ColorPalette(palette)
 
 
-def husl_palette(n_colors=6, h=.01, s=.9, l=.65):
+def husl_palette(n_colors=6, h=.01, s=.9, l=.65):  # noqa
     """Get a set of evenly spaced colors in HUSL hue space.
 
     h, s, and l should be between 0 and 1
@@ -323,13 +376,13 @@ def husl_palette(n_colors=6, h=.01, s=.9, l=.65):
         >>> sns.palplot(sns.husl_palette(10, s=.4))
 
     """
-    hues = np.linspace(0, 1, n_colors + 1)[:-1]
+    hues = np.linspace(0, 1, int(n_colors) + 1)[:-1]
     hues += h
     hues %= 1
     hues *= 359
     s *= 99
-    l *= 99
-    palette = [husl.husl_to_rgb(h_i, s, l) for h_i in hues]
+    l *= 99  # noqa
+    palette = [_color_to_rgb((h_i, s, l), input='husl') for h_i in hues]
     return _ColorPalette(palette)
 
 
@@ -393,11 +446,6 @@ def mpl_palette(name, n_colors=6):
         >>> sns.palplot(sns.mpl_palette("GnBu_d"))
 
     """
-    mpl_qual_pals = {"Accent": 8, "Dark2": 8, "Paired": 12,
-                     "Pastel1": 9, "Pastel2": 8,
-                     "Set1": 9, "Set2": 8, "Set3": 12,
-                     "tab10": 10, "tab20": 20, "tab20b": 20, "tab20c": 20}
-
     if name.endswith("_d"):
         pal = ["#333333"]
         pal.extend(color_palette(name.replace("_d", "_r"), 2))
@@ -406,10 +454,10 @@ def mpl_palette(name, n_colors=6):
         cmap = mpl.cm.get_cmap(name)
         if cmap is None:
             raise ValueError("{} is not a valid colormap".format(name))
-    if name in mpl_qual_pals:
-        bins = np.linspace(0, 1, mpl_qual_pals[name])[:n_colors]
+    if name in MPL_QUAL_PALS:
+        bins = np.linspace(0, 1, MPL_QUAL_PALS[name])[:n_colors]
     else:
-        bins = np.linspace(0, 1, n_colors + 2)[1:-1]
+        bins = np.linspace(0, 1, int(n_colors) + 2)[1:-1]
     palette = list(map(tuple, cmap(bins)[:, :3]))
 
     return _ColorPalette(palette)
@@ -421,8 +469,10 @@ def _color_to_rgb(color, input):
         color = colorsys.hls_to_rgb(*color)
     elif input == "husl":
         color = husl.husl_to_rgb(*color)
+        color = tuple(np.clip(color, 0, 1))
     elif input == "xkcd":
         color = xkcd_rgb[color]
+
     return color
 
 
@@ -586,7 +636,7 @@ def light_palette(color, n_colors=6, reverse=False, as_cmap=False,
 
     """
     color = _color_to_rgb(color, input)
-    light = set_hls_values(color, l=.95)
+    light = set_hls_values(color, l=.95)  # noqa
     colors = [color, light] if reverse else [light, color]
     return blend_palette(colors, n_colors, as_cmap)
 
@@ -618,8 +668,8 @@ def _flat_palette(color, n_colors=6, reverse=False, as_cmap=False,
     return blend_palette(colors, n_colors, as_cmap)
 
 
-def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6, center="light",
-                      as_cmap=False):
+def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6,  # noqa
+                      center="light", as_cmap=False):
     """Make a diverging palette between two HUSL colors.
 
     If you are using the IPython notebook, you can also choose this palette
@@ -633,6 +683,8 @@ def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6, center="light",
         Anchor saturation for both extents of the map.
     l : float in [0, 100], optional
         Anchor lightness for both extents of the map.
+    sep : int, optional
+        Size of the intermediate region.
     n : int, optional
         Number of colors in the palette (if not returning a cmap)
     center : {"light", "dark"}, optional
@@ -691,10 +743,10 @@ def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6, center="light",
 
     """
     palfunc = dark_palette if center == "dark" else light_palette
-    neg = palfunc((h_neg, s, l), 128 - (sep / 2), reverse=True, input="husl")
-    pos = palfunc((h_pos, s, l), 128 - (sep / 2), input="husl")
-    midpoint = dict(light=[(.95, .95, .95, 1.)],
-                    dark=[(.133, .133, .133, 1.)])[center]
+    n_half = int(128 - (sep // 2))
+    neg = palfunc((h_neg, s, l), n_half, reverse=True, input="husl")
+    pos = palfunc((h_pos, s, l), n_half, input="husl")
+    midpoint = dict(light=[(.95, .95, .95)], dark=[(.133, .133, .133)])[center]
     mid = midpoint * sep
     pal = blend_palette(np.concatenate([neg, mid,  pos]), n, as_cmap=as_cmap)
     return pal
@@ -724,14 +776,15 @@ def blend_palette(colors, n_colors=6, as_cmap=False, input="rgb"):
     name = "blend"
     pal = mpl.colors.LinearSegmentedColormap.from_list(name, colors)
     if not as_cmap:
-        pal = _ColorPalette(pal(np.linspace(0, 1, n_colors)))
+        rgb_array = pal(np.linspace(0, 1, int(n_colors)))[:, :3]  # no alpha
+        pal = _ColorPalette(map(tuple, rgb_array))
     return pal
 
 
 def xkcd_palette(colors):
     """Make a palette with color names from the xkcd color survey.
 
-    See xkcd for the full list of colors: http://xkcd.com/color/rgb/
+    See xkcd for the full list of colors: https://xkcd.com/color/rgb/
 
     This is just a simple wrapper around the ``seaborn.xkcd_rgb`` dictionary.
 
@@ -759,7 +812,7 @@ def crayon_palette(colors):
     """Make a palette with color names from Crayola crayons.
 
     Colors are taken from here:
-    http://en.wikipedia.org/wiki/List_of_Crayola_crayon_colors
+    https://en.wikipedia.org/wiki/List_of_Crayola_crayon_colors
 
     This is just a simple wrapper around the ``seaborn.crayons`` dictionary.
 
@@ -794,6 +847,10 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
     user more control over the look of the palette and has a different set of
     defaults.
 
+    In addition to using this function, it is also possible to generate a
+    cubehelix palette generally in seaborn using a string-shorthand; see the
+    example below.
+
     Parameters
     ----------
     n_colors : int
@@ -886,11 +943,39 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
         >>> cmap = sns.cubehelix_palette(dark=0, light=1, as_cmap=True)
         >>> ax = sns.heatmap(x, cmap=cmap)
 
+    Use through the :func:`color_palette` interface:
+
+    .. plot::
+        :context: close-figs
+
+        >>> sns.palplot(sns.color_palette("ch:2,r=.2,l=.6"))
+
     """
-    cdict = mpl._cm.cubehelix(gamma, start, rot, hue)
+    def get_color_function(p0, p1):
+        # Copied from matplotlib because it lives in private module
+        def color(x):
+            # Apply gamma factor to emphasise low or high intensity values
+            xg = x ** gamma
+
+            # Calculate amplitude and angle of deviation from the black
+            # to white diagonal in the plane of constant
+            # perceived intensity.
+            a = hue * xg * (1 - xg) / 2
+
+            phi = 2 * np.pi * (start / 3 + rot * x)
+
+            return xg + a * (p0 * np.cos(phi) + p1 * np.sin(phi))
+        return color
+
+    cdict = {
+            "red": get_color_function(-0.14861, 1.78277),
+            "green": get_color_function(-0.29227, -0.90649),
+            "blue": get_color_function(1.97294, 0.0),
+    }
+
     cmap = mpl.colors.LinearSegmentedColormap("cubehelix", cdict)
 
-    x = np.linspace(light, dark, n_colors)
+    x = np.linspace(light, dark, int(n_colors))
     pal = cmap(x)[:, :3].tolist()
     if reverse:
         pal = pal[::-1]
@@ -900,12 +985,47 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
         if reverse:
             x_256 = x_256[::-1]
         pal_256 = cmap(x_256)
-        cmap = mpl.colors.ListedColormap(pal_256)
+        cmap = mpl.colors.ListedColormap(pal_256, "seaborn_cubehelix")
         return cmap
     else:
         return _ColorPalette(pal)
 
 
+def _parse_cubehelix_args(argstr):
+    """Turn stringified cubehelix params into args/kwargs."""
+
+    if argstr.startswith("ch:"):
+        argstr = argstr[3:]
+
+    if argstr.endswith("_r"):
+        reverse = True
+        argstr = argstr[:-2]
+    else:
+        reverse = False
+
+    if not argstr:
+        return [], {"reverse": reverse}
+
+    all_args = argstr.split(",")
+
+    args = [float(a.strip(" ")) for a in all_args if "=" not in a]
+
+    kwargs = [a.split("=") for a in all_args if "=" in a]
+    kwargs = {k.strip(" "): float(v.strip(" ")) for k, v in kwargs}
+
+    kwarg_map = dict(
+        s="start", r="rot", g="gamma",
+        h="hue", l="light", d="dark",  # noqa: E741
+    )
+
+    kwargs = {kwarg_map.get(k, k): v for k, v in kwargs.items()}
+
+    if reverse:
+        kwargs["reverse"] = True
+
+    return args, kwargs
+
+
 def set_color_codes(palette="deep"):
     """Change how matplotlib color shorthands are interpreted.
 
@@ -948,10 +1068,19 @@ def set_color_codes(palette="deep"):
 
     """
     if palette == "reset":
-        colors = [(0., 0., 1.), (0., .5, 0.), (1., 0., 0.), (.75, .75, 0.),
+        colors = [(0., 0., 1.), (0., .5, 0.), (1., 0., 0.), (.75, 0., .75),
                   (.75, .75, 0.), (0., .75, .75), (0., 0., 0.)]
-    else:
+    elif not isinstance(palette, str):
+        err = "set_color_codes requires a named seaborn palette"
+        raise TypeError(err)
+    elif palette in SEABORN_PALETTES:
+        if not palette.endswith("6"):
+            palette = palette + "6"
         colors = SEABORN_PALETTES[palette] + [(.1, .1, .1)]
+    else:
+        err = "Cannot set colors with palette '{}'".format(palette)
+        raise ValueError(err)
+
     for code, color in zip("bgrmyck", colors):
         rgb = mpl.colors.colorConverter.to_rgb(color)
         mpl.colors.colorConverter.colors[code] = rgb
