@@ -1,16 +1,13 @@
 """Plotting functions for linear models (broadly construed)."""
-from __future__ import division
 import copy
-import itertools
 from textwrap import dedent
+import warnings
 import numpy as np
 import pandas as pd
 from scipy.spatial import distance
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 
-import warnings
-
 try:
     import statsmodels
     assert statsmodels
@@ -18,12 +15,8 @@ try:
 except ImportError:
     _has_statsmodels = False
 
-from .external.six import string_types
-from .external.six.moves import range
-
 from . import utils
 from . import algorithms as algo
-from .palettes import color_palette
 from .axisgrid import FacetGrid, _facet_docs
 
 
@@ -42,16 +35,24 @@ class _LinearPlotter(object):
         self.data = data
 
         # Validate the inputs
-        any_strings = any([isinstance(v, string_types) for v in kws.values()])
+        any_strings = any([isinstance(v, str) for v in kws.values()])
         if any_strings and data is None:
             raise ValueError("Must pass `data` if using named variables.")
 
         # Set the variables
         for var, val in kws.items():
-            if isinstance(val, string_types):
-                setattr(self, var, data[val])
+            if isinstance(val, str):
+                vector = data[val]
+            elif isinstance(val, list):
+                vector = np.asarray(val)
             else:
-                setattr(self, var, val)
+                vector = val
+            if vector is not None and vector.shape != (1,):
+                vector = np.squeeze(vector)
+            if np.ndim(vector) > 1:
+                err = "regplot inputs must be 1d"
+                raise ValueError(err)
+            setattr(self, var, vector)
 
     def dropna(self, *vars):
         """Remove observations with missing data."""
@@ -75,7 +76,7 @@ class _RegressionPlotter(_LinearPlotter):
     """
     def __init__(self, x, y, data=None, x_estimator=None, x_bins=None,
                  x_ci="ci", scatter=True, fit_reg=True, ci=95, n_boot=1000,
-                 units=None, order=1, logistic=False, lowess=False,
+                 units=None, seed=None, order=1, logistic=False, lowess=False,
                  robust=False, logx=False, x_partial=None, y_partial=None,
                  truncate=False, dropna=True, x_jitter=None, y_jitter=None,
                  color=None, label=None):
@@ -85,6 +86,7 @@ class _RegressionPlotter(_LinearPlotter):
         self.ci = ci
         self.x_ci = ci if x_ci == "ci" else x_ci
         self.n_boot = n_boot
+        self.seed = seed
         self.scatter = scatter
         self.fit_reg = fit_reg
         self.order = order
@@ -124,8 +126,13 @@ class _RegressionPlotter(_LinearPlotter):
         else:
             self.x_discrete = self.x
 
+        # Disable regression in case of singleton inputs
+        if len(self.x) <= 1:
+            self.fit_reg = False
+
         # Save the range of the x variable for the grid later
-        self.x_range = self.x.min(), self.x.max()
+        if self.fit_reg:
+            self.x_range = self.x.min(), self.x.max()
 
     @property
     def scatter_data(self):
@@ -169,8 +176,11 @@ class _RegressionPlotter(_LinearPlotter):
                 else:
                     if self.units is not None:
                         units = self.units[x == val]
-                    boots = algo.bootstrap(_y, func=self.x_estimator,
-                                           n_boot=self.n_boot, units=units)
+                    boots = algo.bootstrap(_y,
+                                           func=self.x_estimator,
+                                           n_boot=self.n_boot,
+                                           units=units,
+                                           seed=self.seed)
                     _ci = utils.ci(boots, self.x_ci)
                 cis.append(_ci)
 
@@ -219,35 +229,45 @@ class _RegressionPlotter(_LinearPlotter):
 
     def fit_fast(self, grid):
         """Low-level regression and prediction using linear algebra."""
+        def reg_func(_x, _y):
+            return np.linalg.pinv(_x).dot(_y)
+
         X, y = np.c_[np.ones(len(self.x)), self.x], self.y
         grid = np.c_[np.ones(len(grid)), grid]
-        reg_func = lambda _x, _y: np.linalg.pinv(_x).dot(_y)
         yhat = grid.dot(reg_func(X, y))
         if self.ci is None:
             return yhat, None
 
-        beta_boots = algo.bootstrap(X, y, func=reg_func,
-                                    n_boot=self.n_boot, units=self.units).T
+        beta_boots = algo.bootstrap(X, y,
+                                    func=reg_func,
+                                    n_boot=self.n_boot,
+                                    units=self.units,
+                                    seed=self.seed).T
         yhat_boots = grid.dot(beta_boots).T
         return yhat, yhat_boots
 
     def fit_poly(self, grid, order):
         """Regression using numpy polyfit for higher-order trends."""
+        def reg_func(_x, _y):
+            return np.polyval(np.polyfit(_x, _y, order), grid)
+
         x, y = self.x, self.y
-        reg_func = lambda _x, _y: np.polyval(np.polyfit(_x, _y, order), grid)
         yhat = reg_func(x, y)
         if self.ci is None:
             return yhat, None
 
-        yhat_boots = algo.bootstrap(x, y, func=reg_func,
-                                    n_boot=self.n_boot, units=self.units)
+        yhat_boots = algo.bootstrap(x, y,
+                                    func=reg_func,
+                                    n_boot=self.n_boot,
+                                    units=self.units,
+                                    seed=self.seed)
         return yhat, yhat_boots
 
     def fit_statsmodels(self, grid, model, **kwargs):
         """More general regression function using statsmodels objects."""
+        import statsmodels.genmod.generalized_linear_model as glm
         X, y = np.c_[np.ones(len(self.x)), self.x], self.y
         grid = np.c_[np.ones(len(grid)), grid]
-        import statsmodels.genmod.generalized_linear_model as glm
 
         def reg_func(_x, _y):
             try:
@@ -261,8 +281,11 @@ class _RegressionPlotter(_LinearPlotter):
         if self.ci is None:
             return yhat, None
 
-        yhat_boots = algo.bootstrap(X, y, func=reg_func,
-                                    n_boot=self.n_boot, units=self.units)
+        yhat_boots = algo.bootstrap(X, y,
+                                    func=reg_func,
+                                    n_boot=self.n_boot,
+                                    units=self.units,
+                                    seed=self.seed)
         return yhat, yhat_boots
 
     def fit_lowess(self):
@@ -284,8 +307,11 @@ class _RegressionPlotter(_LinearPlotter):
         if self.ci is None:
             return yhat, None
 
-        beta_boots = algo.bootstrap(X, y, func=reg_func,
-                                    n_boot=self.n_boot, units=self.units).T
+        beta_boots = algo.bootstrap(X, y,
+                                    func=reg_func,
+                                    n_boot=self.n_boot,
+                                    units=self.units,
+                                    seed=self.seed).T
         yhat_boots = grid.dot(beta_boots).T
         return yhat, yhat_boots
 
@@ -294,7 +320,7 @@ class _RegressionPlotter(_LinearPlotter):
         x = self.x
         if np.isscalar(bins):
             percentiles = np.linspace(0, 100, bins + 2)[1:-1]
-            bins = np.c_[utils.percentiles(x, percentiles)]
+            bins = np.c_[np.percentile(x, percentiles)]
         else:
             bins = np.c_[np.ravel(bins)]
 
@@ -310,7 +336,7 @@ class _RegressionPlotter(_LinearPlotter):
         b = b - b.mean()
         b = np.c_[b]
         a_prime = a - b.dot(np.linalg.pinv(b).dot(a))
-        return (a_prime + a_mean).reshape(a.shape)
+        return np.asarray(a_prime + a_mean).reshape(a.shape)
 
     def plot(self, ax, scatter_kws, line_kws):
         """Draw the full plot."""
@@ -322,13 +348,13 @@ class _RegressionPlotter(_LinearPlotter):
 
         # Use the current color cycle state as a default
         if self.color is None:
-            lines, = plt.plot(self.x.mean(), self.y.mean())
+            lines, = ax.plot([], [])
             color = lines.get_color()
             lines.remove()
         else:
             color = self.color
 
-        # Ensure that color is hex to avoid matplotlib weidness
+        # Ensure that color is hex to avoid matplotlib weirdness
         color = mpl.colors.rgb2hex(mpl.colors.colorConverter.to_rgb(color))
 
         # Let color in keyword arguments override overall plot color
@@ -338,6 +364,7 @@ class _RegressionPlotter(_LinearPlotter):
         # Draw the constituent plots
         if self.scatter:
             self.scatterplot(ax, scatter_kws)
+
         if self.fit_reg:
             self.lineplot(ax, line_kws)
 
@@ -381,10 +408,9 @@ class _RegressionPlotter(_LinearPlotter):
 
     def lineplot(self, ax, kws):
         """Draw the model."""
-        xlim = ax.get_xlim()
-
         # Fit the regression model
         grid, yhat, err_bands = self.fit_regression(ax)
+        edges = grid[0], grid[-1]
 
         # Get set default aesthetics
         fill_color = kws["color"]
@@ -392,35 +418,24 @@ class _RegressionPlotter(_LinearPlotter):
         kws.setdefault("linewidth", lw)
 
         # Draw the regression line and confidence interval
-        ax.plot(grid, yhat, **kws)
+        line, = ax.plot(grid, yhat, **kws)
+        line.sticky_edges.x[:] = edges  # Prevent mpl from adding margin
         if err_bands is not None:
             ax.fill_between(grid, *err_bands, facecolor=fill_color, alpha=.15)
-        ax.set_xlim(*xlim)
 
 
 _regression_docs = dict(
 
     model_api=dedent("""\
     There are a number of mutually exclusive options for estimating the
-    regression model: ``order``, ``logistic``, ``lowess``, ``robust``, and
-    ``logx``. See the parameter docs for more information on these options.\
+    regression model. See the :ref:`tutorial <regression_tutorial>` for more
+    information.\
     """),
-
     regplot_vs_lmplot=dedent("""\
-    Understanding the difference between :func:`regplot` and :func:`lmplot` can
-    be a bit tricky. In fact, they are closely related, as :func:`lmplot` uses
-    :func:`regplot` internally and takes most of its parameters. However,
-    :func:`regplot` is an axes-level function, so it draws directly onto an
-    axes (either the currently active axes or the one provided by the ``ax``
-    parameter), while :func:`lmplot` is a figure-level function and creates its
-    own figure, which is managed through a :class:`FacetGrid`. This has a few
-    consequences, namely that :func:`regplot` can happily coexist in a figure
-    with other kinds of plots and will follow the global matplotlib color
-    cycle. In contrast, :func:`lmplot` needs to occupy an entire figure, and
-    the size and color cycle are controlled through function parameters,
-    ignoring the global defaults.\
+    The :func:`regplot` and :func:`lmplot` functions are closely related, but
+    the former is an axes-level function while the latter is a figure-level
+    function that combines :func:`regplot` and :class:`FacetGrid`.\
     """),
-
     x_estimator=dedent("""\
     x_estimator : callable that maps vector -> scalar, optional
         Apply this function to each unique value of ``x`` and plot the
@@ -442,8 +457,8 @@ _regression_docs = dict(
     x_ci : "ci", "sd", int in [0, 100] or None, optional
         Size of the confidence interval used when plotting a central tendency
         for discrete values of ``x``. If ``"ci"``, defer to the value of the
-        ``ci`` parameter. If ``"sd"``, skip bootstrappig and show the standard
-        deviation of the observations in each bin.\
+        ``ci`` parameter. If ``"sd"``, skip bootstrapping and show the
+        standard deviation of the observations in each bin.\
     """),
     scatter=dedent("""\
     scatter : bool, optional
@@ -477,6 +492,10 @@ _regression_docs = dict(
         that resamples both units and observations (within unit). This does not
         otherwise influence how the regression is estimated or drawn.\
     """),
+    seed=dedent("""\
+    seed : int, numpy.random.Generator, or numpy.random.RandomState, optional
+        Seed or random number generator for reproducible bootstrapping.\
+    """),
     order=dedent("""\
     order : int, optional
         If ``order`` is greater than 1, use ``numpy.polyfit`` to estimate a
@@ -517,9 +536,8 @@ _regression_docs = dict(
     """),
     truncate=dedent("""\
     truncate : bool, optional
-        By default, the regression line is drawn to fill the x axis limits
-        after the scatterplot is drawn. If ``truncate`` is ``True``, it will
-        instead by bounded by the data limits.\
+        If ``True``, the regression line is bounded by the data limits. If
+        ``False``, it extends to the ``x`` axis limits.
     """),
     xy_jitter=dedent("""\
     {x,y}_jitter : floats, optional
@@ -538,13 +556,21 @@ _regression_docs.update(_facet_docs)
 
 
 def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
-           col_wrap=None, size=5, aspect=1, markers="o", sharex=True,
+           col_wrap=None, height=5, aspect=1, markers="o", sharex=True,
            sharey=True, hue_order=None, col_order=None, row_order=None,
            legend=True, legend_out=True, x_estimator=None, x_bins=None,
            x_ci="ci", scatter=True, fit_reg=True, ci=95, n_boot=1000,
-           units=None, order=1, logistic=False, lowess=False, robust=False,
-           logx=False, x_partial=None, y_partial=None, truncate=False,
-           x_jitter=None, y_jitter=None, scatter_kws=None, line_kws=None):
+           units=None, seed=None, order=1, logistic=False, lowess=False,
+           robust=False, logx=False, x_partial=None, y_partial=None,
+           truncate=True, x_jitter=None, y_jitter=None, scatter_kws=None,
+           line_kws=None, size=None):
+
+    # Handle deprecations
+    if size is not None:
+        height = size
+        msg = ("The `size` parameter has been renamed to `height`; "
+               "please update your code.")
+        warnings.warn(msg, UserWarning)
 
     # Reduce the dataframe to only needed columns
     need_cols = [x, y, hue, col, row, units, x_partial, y_partial]
@@ -554,7 +580,7 @@ def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
     # Initialize the grid
     facets = FacetGrid(data, row, col, hue, palette=palette,
                        row_order=row_order, col_order=col_order,
-                       hue_order=hue_order, size=size, aspect=aspect,
+                       hue_order=hue_order, height=height, aspect=aspect,
                        col_wrap=col_wrap, sharex=sharex, sharey=sharey,
                        legend_out=legend_out)
 
@@ -582,9 +608,9 @@ def lmplot(x, y, data, hue=None, col=None, row=None, palette=None,
     regplot_kws = dict(
         x_estimator=x_estimator, x_bins=x_bins, x_ci=x_ci,
         scatter=scatter, fit_reg=fit_reg, ci=ci, n_boot=n_boot, units=units,
-        order=order, logistic=logistic, lowess=lowess, robust=robust,
-        logx=logx, x_partial=x_partial, y_partial=y_partial, truncate=truncate,
-        x_jitter=x_jitter, y_jitter=y_jitter,
+        seed=seed, order=order, logistic=logistic, lowess=lowess,
+        robust=robust, logx=logx, x_partial=x_partial, y_partial=y_partial,
+        truncate=truncate, x_jitter=x_jitter, y_jitter=y_jitter,
         scatter_kws=scatter_kws, line_kws=line_kws,
         )
     facets.map_dataframe(regplot, x, y, **regplot_kws)
@@ -625,7 +651,7 @@ lmplot.__doc__ = dedent("""\
         the order of levels of this variable.
     {palette}
     {col_wrap}
-    {size}
+    {height}
     {aspect}
     markers : matplotlib marker code or list of marker codes, optional
         Markers for the scatterplot. If a list, each marker in the list will be
@@ -646,6 +672,7 @@ lmplot.__doc__ = dedent("""\
     {ci}
     {n_boot}
     {units}
+    {seed}
     {order}
     {logistic}
     {lowess}
@@ -725,13 +752,13 @@ lmplot.__doc__ = dedent("""\
 
         >>> g = sns.lmplot(x="total_bill", y="tip", col="smoker", data=tips)
 
-    Change the size and aspect ratio of the facets:
+    Change the height and aspect ratio of the facets:
 
     .. plot::
         :context: close-figs
 
         >>> g = sns.lmplot(x="size", y="total_bill", hue="day", col="day",
-        ...                data=tips, aspect=.4, x_jitter=.1)
+        ...                data=tips, height=6, aspect=.4, x_jitter=.1)
 
     Wrap the levels of the column variable into multiple rows:
 
@@ -739,7 +766,7 @@ lmplot.__doc__ = dedent("""\
         :context: close-figs
 
         >>> g = sns.lmplot(x="total_bill", y="tip", col="day", hue="day",
-        ...                data=tips, col_wrap=2, size=3)
+        ...                data=tips, col_wrap=2, height=3)
 
     Condition on two variables to make a full grid:
 
@@ -747,7 +774,7 @@ lmplot.__doc__ = dedent("""\
         :context: close-figs
 
         >>> g = sns.lmplot(x="total_bill", y="tip", row="sex", col="time",
-        ...                data=tips, size=3)
+        ...                data=tips, height=3)
 
     Use methods on the returned :class:`FacetGrid` instance to further tweak
     the plot:
@@ -756,7 +783,7 @@ lmplot.__doc__ = dedent("""\
         :context: close-figs
 
         >>> g = sns.lmplot(x="total_bill", y="tip", row="sex", col="time",
-        ...                data=tips, size=3)
+        ...                data=tips, height=3)
         >>> g = (g.set_axis_labels("Total bill (US Dollars)", "Tip")
         ...       .set(xlim=(0, 60), ylim=(0, 12),
         ...            xticks=[10, 30, 50], yticks=[2, 6, 10])
@@ -769,14 +796,14 @@ lmplot.__doc__ = dedent("""\
 
 def regplot(x, y, data=None, x_estimator=None, x_bins=None, x_ci="ci",
             scatter=True, fit_reg=True, ci=95, n_boot=1000, units=None,
-            order=1, logistic=False, lowess=False, robust=False,
+            seed=None, order=1, logistic=False, lowess=False, robust=False,
             logx=False, x_partial=None, y_partial=None,
-            truncate=False, dropna=True, x_jitter=None, y_jitter=None,
+            truncate=True, dropna=True, x_jitter=None, y_jitter=None,
             label=None, color=None, marker="o",
             scatter_kws=None, line_kws=None, ax=None):
 
     plotter = _RegressionPlotter(x, y, data, x_estimator, x_bins, x_ci,
-                                 scatter, fit_reg, ci, n_boot, units,
+                                 scatter, fit_reg, ci, n_boot, units, seed,
                                  order, logistic, lowess, robust, logx,
                                  x_partial, y_partial, truncate, dropna,
                                  x_jitter, y_jitter, color, label)
@@ -790,6 +817,7 @@ def regplot(x, y, data=None, x_estimator=None, x_bins=None, x_ci="ci",
     plotter.plot(ax, scatter_kws, line_kws)
     return ax
 
+
 regplot.__doc__ = dedent("""\
     Plot data and a linear regression model fit.
 
@@ -810,6 +838,7 @@ regplot.__doc__ = dedent("""\
     {ci}
     {n_boot}
     {units}
+    {seed}
     {order}
     {logistic}
     {lowess}
@@ -819,7 +848,7 @@ regplot.__doc__ = dedent("""\
     {truncate}
     {xy_jitter}
     label : string
-        Label to apply to ether the scatterplot or regression line (if
+        Label to apply to either the scatterplot or regression line (if
         ``scatter`` is ``False``) for use in a legend.
     color : matplotlib color
         Color to apply to all plot elements; will be superseded by colors
@@ -888,12 +917,12 @@ regplot.__doc__ = dedent("""\
         >>> ax = sns.regplot(x=x, y=y, marker="+")
 
     Use a 68% confidence interval, which corresponds with the standard error
-    of the estimate:
+    of the estimate, and extend the regression line to the axis limits:
 
     .. plot::
         :context: close-figs
 
-        >>> ax = sns.regplot(x=x, y=y, ci=68)
+        >>> ax = sns.regplot(x=x, y=y, ci=68, truncate=False)
 
     Plot with a discrete ``x`` variable and add some jitter:
 
@@ -918,7 +947,7 @@ regplot.__doc__ = dedent("""\
 
         >>> ax = sns.regplot(x=x, y=y, x_bins=4)
 
-    Fit a higher-order polynomial regression and truncate the model prediction:
+    Fit a higher-order polynomial regression:
 
     .. plot::
         :context: close-figs
@@ -926,7 +955,7 @@ regplot.__doc__ = dedent("""\
         >>> ans = sns.load_dataset("anscombe")
         >>> ax = sns.regplot(x="x", y="y", data=ans.loc[ans.dataset == "II"],
         ...                  scatter_kws={{"s": 80}},
-        ...                  order=2, ci=None, truncate=True)
+        ...                  order=2, ci=None)
 
     Fit a robust regression and don't plot a confidence interval:
 
@@ -947,13 +976,13 @@ regplot.__doc__ = dedent("""\
         >>> ax = sns.regplot(x="total_bill", y="big_tip", data=tips,
         ...                  logistic=True, n_boot=500, y_jitter=.03)
 
-    Fit the regression model using log(x) and truncate the model prediction:
+    Fit the regression model using log(x):
 
     .. plot::
         :context: close-figs
 
         >>> ax = sns.regplot(x="size", y="total_bill", data=tips,
-        ...                  x_estimator=np.mean, logx=True, truncate=True)
+        ...                  x_estimator=np.mean, logx=True)
 
     """).format(**_regression_docs)
 
@@ -1008,8 +1037,8 @@ def residplot(x, y, data=None, lowess=False, x_partial=None, y_partial=None,
     See Also
     --------
     regplot : Plot a simple linear regression model.
-    jointplot (with kind="resid"): Draw a residplot with univariate
-                                   marginal distrbutions.
+    jointplot : Draw a :func:`residplot` with univariate marginal distributions
+                (when used with ``kind="resid"``).
 
     """
     plotter = _RegressionPlotter(x, y, data, ci=None,
@@ -1034,230 +1063,7 @@ def residplot(x, y, data=None, lowess=False, x_partial=None, y_partial=None,
     ax.axhline(0, ls=":", c=".2")
 
     # Draw the scatterplot
-    scatter_kws = {} if scatter_kws is None else scatter_kws
-    line_kws = {} if line_kws is None else line_kws
+    scatter_kws = {} if scatter_kws is None else scatter_kws.copy()
+    line_kws = {} if line_kws is None else line_kws.copy()
     plotter.plot(ax, scatter_kws, line_kws)
     return ax
-
-
-def coefplot(formula, data, groupby=None, intercept=False, ci=95,
-             palette="husl"):
-    """Plot the coefficients from a linear model.
-
-    Parameters
-    ----------
-    formula : string
-        patsy formula for ols model
-    data : dataframe
-        data for the plot; formula terms must appear in columns
-    groupby : grouping object, optional
-        object to group data with to fit conditional models
-    intercept : bool, optional
-        if False, strips the intercept term before plotting
-    ci : float, optional
-        size of confidence intervals
-    palette : seaborn color palette, optional
-        palette for the horizonal plots
-
-    """
-    msg = (
-        "The `coefplot` function has been deprecated and will be removed "
-        "in a future version."
-    )
-    warnings.warn(msg, UserWarning)
-    if not _has_statsmodels:
-        raise ImportError("The `coefplot` function requires statsmodels")
-    import statsmodels.formula.api as sf
-
-    alpha = 1 - ci / 100
-    if groupby is None:
-        coefs = sf.ols(formula, data).fit().params
-        cis = sf.ols(formula, data).fit().conf_int(alpha)
-    else:
-        grouped = data.groupby(groupby)
-        coefs = grouped.apply(lambda d: sf.ols(formula, d).fit().params).T
-        cis = grouped.apply(lambda d: sf.ols(formula, d).fit().conf_int(alpha))
-
-    # Possibly ignore the intercept
-    if not intercept:
-        coefs = coefs.ix[1:]
-
-    n_terms = len(coefs)
-
-    # Plot seperately depending on groupby
-    w, h = mpl.rcParams["figure.figsize"]
-    hsize = lambda n: n * (h / 2)
-    wsize = lambda n: n * (w / (4 * (n / 5)))
-    if groupby is None:
-        colors = itertools.cycle(color_palette(palette, n_terms))
-        f, ax = plt.subplots(1, 1, figsize=(wsize(n_terms), hsize(1)))
-        for i, term in enumerate(coefs.index):
-            color = next(colors)
-            low, high = cis.ix[term]
-            ax.plot([i, i], [low, high], c=color,
-                    solid_capstyle="round", lw=2.5)
-            ax.plot(i, coefs.ix[term], "o", c=color, ms=8)
-        ax.set_xlim(-.5, n_terms - .5)
-        ax.axhline(0, ls="--", c="dimgray")
-        ax.set_xticks(range(n_terms))
-        ax.set_xticklabels(coefs.index)
-
-    else:
-        n_groups = len(coefs.columns)
-        f, axes = plt.subplots(n_terms, 1, sharex=True,
-                               figsize=(wsize(n_groups), hsize(n_terms)))
-        if n_terms == 1:
-            axes = [axes]
-        colors = itertools.cycle(color_palette(palette, n_groups))
-        for ax, term in zip(axes, coefs.index):
-            for i, group in enumerate(coefs.columns):
-                color = next(colors)
-                low, high = cis.ix[(group, term)]
-                ax.plot([i, i], [low, high], c=color,
-                        solid_capstyle="round", lw=2.5)
-                ax.plot(i, coefs.loc[term, group], "o", c=color, ms=8)
-            ax.set_xlim(-.5, n_groups - .5)
-            ax.axhline(0, ls="--", c="dimgray")
-            ax.set_title(term)
-        ax.set_xlabel(groupby)
-        ax.set_xticks(range(n_groups))
-        ax.set_xticklabels(coefs.columns)
-
-
-def interactplot(x1, x2, y, data=None, filled=False, cmap="RdBu_r",
-                 colorbar=True, levels=30, logistic=False,
-                 contour_kws=None, scatter_kws=None, ax=None, **kwargs):
-    """Visualize a continuous two-way interaction with a contour plot.
-
-    Parameters
-    ----------
-    x1, x2, y, strings or array-like
-        Either the two independent variables and the dependent variable,
-        or keys to extract them from `data`
-    data : DataFrame
-        Pandas DataFrame with the data in the columns.
-    filled : bool
-        Whether to plot with filled or unfilled contours
-    cmap : matplotlib colormap
-        Colormap to represent yhat in the countour plot.
-    colorbar : bool
-        Whether to draw the colorbar for interpreting the color values.
-    levels : int or sequence
-        Number or position of contour plot levels.
-    logistic : bool
-        Fit a logistic regression model instead of linear regression.
-    contour_kws : dictionary
-        Keyword arguments for contour[f]().
-    scatter_kws : dictionary
-        Keyword arguments for plot().
-    ax : matplotlib axis
-        Axis to draw plot in.
-
-    Returns
-    -------
-    ax : Matplotlib axis
-        Axis with the contour plot.
-
-    """
-    msg = (
-        "The `interactplot` function has been deprecated and will be removed "
-        "in a future version."
-    )
-    warnings.warn(msg, UserWarning)
-    if not _has_statsmodels:
-        raise ImportError("The `interactplot` function requires statsmodels")
-    from statsmodels.regression.linear_model import OLS
-    from statsmodels.genmod.generalized_linear_model import GLM
-    from statsmodels.genmod.families import Binomial
-
-    # Handle the form of the data
-    if data is not None:
-        x1 = data[x1]
-        x2 = data[x2]
-        y = data[y]
-    if hasattr(x1, "name"):
-        xlabel = x1.name
-    else:
-        xlabel = None
-    if hasattr(x2, "name"):
-        ylabel = x2.name
-    else:
-        ylabel = None
-    if hasattr(y, "name"):
-        clabel = y.name
-    else:
-        clabel = None
-    x1 = np.asarray(x1)
-    x2 = np.asarray(x2)
-    y = np.asarray(y)
-
-    # Initialize the scatter keyword dictionary
-    if scatter_kws is None:
-        scatter_kws = {}
-    if not ("color" in scatter_kws or "c" in scatter_kws):
-        scatter_kws["color"] = "#222222"
-    if "alpha" not in scatter_kws:
-        scatter_kws["alpha"] = 0.75
-
-    # Intialize the contour keyword dictionary
-    if contour_kws is None:
-        contour_kws = {}
-
-    # Initialize the axis
-    if ax is None:
-        ax = plt.gca()
-
-    # Plot once to let matplotlib sort out the axis limits
-    ax.plot(x1, x2, "o", **scatter_kws)
-
-    # Find the plot limits
-    x1min, x1max = ax.get_xlim()
-    x2min, x2max = ax.get_ylim()
-
-    # Make the grid for the contour plot
-    x1_points = np.linspace(x1min, x1max, 100)
-    x2_points = np.linspace(x2min, x2max, 100)
-    xx1, xx2 = np.meshgrid(x1_points, x2_points)
-
-    # Fit the model with an interaction
-    X = np.c_[np.ones(x1.size), x1, x2, x1 * x2]
-    if logistic:
-        lm = GLM(y, X, family=Binomial()).fit()
-    else:
-        lm = OLS(y, X).fit()
-
-    # Evaluate the model on the grid
-    eval = np.vectorize(lambda x1_, x2_: lm.predict([1, x1_, x2_, x1_ * x2_]))
-    yhat = eval(xx1, xx2)
-
-    # Default color limits put the midpoint at mean(y)
-    y_bar = y.mean()
-    c_min = min(np.percentile(y, 2), yhat.min())
-    c_max = max(np.percentile(y, 98), yhat.max())
-    delta = max(c_max - y_bar, y_bar - c_min)
-    c_min, cmax = y_bar - delta, y_bar + delta
-    contour_kws.setdefault("vmin", c_min)
-    contour_kws.setdefault("vmax", c_max)
-
-    # Draw the contour plot
-    func_name = "contourf" if filled else "contour"
-    contour = getattr(ax, func_name)
-    c = contour(xx1, xx2, yhat, levels, cmap=cmap, **contour_kws)
-
-    # Draw the scatter again so it's visible
-    ax.plot(x1, x2, "o", **scatter_kws)
-
-    # Draw a colorbar, maybe
-    if colorbar:
-        bar = plt.colorbar(c)
-
-    # Label the axes
-    if xlabel is not None:
-        ax.set_xlabel(xlabel)
-    if ylabel is not None:
-        ax.set_ylabel(ylabel)
-    if clabel is not None and colorbar:
-        clabel = "P(%s)" % clabel if logistic else clabel
-        bar.set_label(clabel, labelpad=15, rotation=270)
-
-    return ax
