@@ -1,5 +1,6 @@
 import itertools
 import tempfile
+import copy
 
 import numpy as np
 import matplotlib as mpl
@@ -10,13 +11,14 @@ from scipy.cluster import hierarchy
 
 import nose.tools as nt
 import numpy.testing as npt
-import pandas.util.testing as pdt
-from numpy.testing.decorators import skipif
+try:
+    import pandas.testing as pdt
+except ImportError:
+    import pandas.util.testing as pdt
+import pytest
 
-from . import PlotTestCase
 from .. import matrix as mat
 from .. import color_palette
-from ..external.six.moves import range
 
 try:
     import fastcluster
@@ -27,7 +29,7 @@ except ImportError:
     _no_fastcluster = True
 
 
-class TestHeatmap(PlotTestCase):
+class TestHeatmap(object):
     rs = np.random.RandomState(sum(map(ord, "heatmap")))
 
     x_norm = rs.randn(4, 8)
@@ -44,11 +46,11 @@ class TestHeatmap(PlotTestCase):
     def test_ndarray_input(self):
 
         p = mat._HeatMapper(self.x_norm, **self.default_kws)
-        npt.assert_array_equal(p.plot_data, self.x_norm[::-1])
-        pdt.assert_frame_equal(p.data, pd.DataFrame(self.x_norm).ix[::-1])
+        npt.assert_array_equal(p.plot_data, self.x_norm)
+        pdt.assert_frame_equal(p.data, pd.DataFrame(self.x_norm))
 
         npt.assert_array_equal(p.xticklabels, np.arange(8))
-        npt.assert_array_equal(p.yticklabels, np.arange(4)[::-1])
+        npt.assert_array_equal(p.yticklabels, np.arange(4))
 
         nt.assert_equal(p.xlabel, "")
         nt.assert_equal(p.ylabel, "")
@@ -56,11 +58,11 @@ class TestHeatmap(PlotTestCase):
     def test_df_input(self):
 
         p = mat._HeatMapper(self.df_norm, **self.default_kws)
-        npt.assert_array_equal(p.plot_data, self.x_norm[::-1])
-        pdt.assert_frame_equal(p.data, self.df_norm.ix[::-1])
+        npt.assert_array_equal(p.plot_data, self.x_norm)
+        pdt.assert_frame_equal(p.data, self.df_norm)
 
         npt.assert_array_equal(p.xticklabels, np.arange(8))
-        npt.assert_array_equal(p.yticklabels, ["D", "C", "B", "A"])
+        npt.assert_array_equal(p.yticklabels, self.letters.values)
 
         nt.assert_equal(p.xlabel, "")
         nt.assert_equal(p.ylabel, "letters")
@@ -76,12 +78,13 @@ class TestHeatmap(PlotTestCase):
 
         p = mat._HeatMapper(df, **self.default_kws)
 
-        npt.assert_array_equal(p.yticklabels, ["D-4", "C-3", "B-2", "A-1"])
+        combined_tick_labels = ["A-1", "B-2", "C-3", "D-4"]
+        npt.assert_array_equal(p.yticklabels, combined_tick_labels)
         nt.assert_equal(p.ylabel, "letter-number")
 
         p = mat._HeatMapper(df.T, **self.default_kws)
 
-        npt.assert_array_equal(p.xticklabels, ["A-1", "B-2", "C-3", "D-4"])
+        npt.assert_array_equal(p.xticklabels, combined_tick_labels)
         nt.assert_equal(p.xlabel, "letter-number")
 
     def test_mask_input(self):
@@ -92,24 +95,34 @@ class TestHeatmap(PlotTestCase):
         p = mat._HeatMapper(self.x_norm, **kws)
         plot_data = np.ma.masked_where(mask, self.x_norm)
 
-        npt.assert_array_equal(p.plot_data, plot_data[::-1])
+        npt.assert_array_equal(p.plot_data, plot_data)
 
-    def test_default_sequential_vlims(self):
+    def test_mask_limits(self):
+        """Make sure masked cells are not used to calculate extremes"""
+
+        kws = self.default_kws.copy()
+
+        mask = self.x_norm > 0
+        kws['mask'] = mask
+        p = mat._HeatMapper(self.x_norm, **kws)
+
+        assert p.vmax == np.ma.array(self.x_norm, mask=mask).max()
+        assert p.vmin == np.ma.array(self.x_norm, mask=mask).min()
+
+        mask = self.x_norm < 0
+        kws['mask'] = mask
+        p = mat._HeatMapper(self.x_norm, **kws)
+
+        assert p.vmin == np.ma.array(self.x_norm, mask=mask).min()
+        assert p.vmax == np.ma.array(self.x_norm, mask=mask).max()
+
+    def test_default_vlims(self):
 
         p = mat._HeatMapper(self.df_unif, **self.default_kws)
         nt.assert_equal(p.vmin, self.x_unif.min())
         nt.assert_equal(p.vmax, self.x_unif.max())
-        nt.assert_true(not p.divergent)
 
-    def test_default_diverging_vlims(self):
-
-        p = mat._HeatMapper(self.df_norm, **self.default_kws)
-        vlim = max(abs(self.x_norm.min()), abs(self.x_norm.max()))
-        nt.assert_equal(p.vmin, -vlim)
-        nt.assert_equal(p.vmax, vlim)
-        nt.assert_true(p.divergent)
-
-    def test_robust_sequential_vlims(self):
+    def test_robust_vlims(self):
 
         kws = self.default_kws.copy()
         kws["robust"] = True
@@ -133,9 +146,10 @@ class TestHeatmap(PlotTestCase):
         kws = self.default_kws.copy()
         kws["vmin"] = -4
         kws["vmax"] = 5
+        kws["center"] = 0
         p = mat._HeatMapper(self.df_norm, **kws)
 
-        nt.assert_equal(p.vmin, -5)
+        nt.assert_equal(p.vmin, -4)
         nt.assert_equal(p.vmax, 5)
 
     def test_array_with_nans(self):
@@ -169,7 +183,7 @@ class TestHeatmap(PlotTestCase):
         kws = self.default_kws.copy()
         kws["cmap"] = "BuGn"
         p = mat._HeatMapper(self.df_unif, **kws)
-        nt.assert_equal(p.cmap, "BuGn")
+        nt.assert_equal(p.cmap, mpl.cm.BuGn)
 
     def test_centered_vlims(self):
 
@@ -178,8 +192,72 @@ class TestHeatmap(PlotTestCase):
 
         p = mat._HeatMapper(self.df_unif, **kws)
 
-        nt.assert_true(p.divergent)
-        nt.assert_equal(p.vmax - .5, .5 - p.vmin)
+        nt.assert_equal(p.vmin, self.df_unif.values.min())
+        nt.assert_equal(p.vmax, self.df_unif.values.max())
+
+    def test_default_colors(self):
+
+        vals = np.linspace(.2, 1, 9)
+        cmap = mpl.cm.binary
+        ax = mat.heatmap([vals], cmap=cmap)
+        fc = ax.collections[0].get_facecolors()
+        cvals = np.linspace(0, 1, 9)
+        npt.assert_array_almost_equal(fc, cmap(cvals), 2)
+
+    def test_custom_vlim_colors(self):
+
+        vals = np.linspace(.2, 1, 9)
+        cmap = mpl.cm.binary
+        ax = mat.heatmap([vals], vmin=0, cmap=cmap)
+        fc = ax.collections[0].get_facecolors()
+        npt.assert_array_almost_equal(fc, cmap(vals), 2)
+
+    def test_custom_center_colors(self):
+
+        vals = np.linspace(.2, 1, 9)
+        cmap = mpl.cm.binary
+        ax = mat.heatmap([vals], center=.5, cmap=cmap)
+        fc = ax.collections[0].get_facecolors()
+        npt.assert_array_almost_equal(fc, cmap(vals), 2)
+
+    def test_cmap_with_properties(self):
+
+        kws = self.default_kws.copy()
+        cmap = copy.copy(mpl.cm.get_cmap("BrBG"))
+        cmap.set_bad("red")
+        kws["cmap"] = cmap
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(
+            cmap(np.ma.masked_invalid([np.nan])),
+            hm.cmap(np.ma.masked_invalid([np.nan])))
+
+        kws["center"] = 0.5
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(
+            cmap(np.ma.masked_invalid([np.nan])),
+            hm.cmap(np.ma.masked_invalid([np.nan])))
+
+        kws = self.default_kws.copy()
+        cmap = copy.copy(mpl.cm.get_cmap("BrBG"))
+        cmap.set_under("red")
+        kws["cmap"] = cmap
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(cmap(-np.inf), hm.cmap(-np.inf))
+
+        kws["center"] = .5
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(cmap(-np.inf), hm.cmap(-np.inf))
+
+        kws = self.default_kws.copy()
+        cmap = copy.copy(mpl.cm.get_cmap("BrBG"))
+        cmap.set_over("red")
+        kws["cmap"] = cmap
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(cmap(-np.inf), hm.cmap(-np.inf))
+
+        kws["center"] = .5
+        hm = mat._HeatMapper(self.df_unif, **kws)
+        npt.assert_array_equal(cmap(np.inf), hm.cmap(np.inf))
 
     def test_tickabels_off(self):
         kws = self.default_kws.copy()
@@ -197,29 +275,29 @@ class TestHeatmap(PlotTestCase):
         kws['yticklabels'] = yticklabels
         p = mat._HeatMapper(self.df_norm, **kws)
         nt.assert_equal(p.xticklabels, xticklabels)
-        nt.assert_equal(p.yticklabels, yticklabels[::-1])
+        nt.assert_equal(p.yticklabels, yticklabels)
 
     def test_custom_ticklabel_interval(self):
 
         kws = self.default_kws.copy()
-        kws['xticklabels'] = 2
-        kws['yticklabels'] = 3
+        xstep, ystep = 2, 3
+        kws['xticklabels'] = xstep
+        kws['yticklabels'] = ystep
         p = mat._HeatMapper(self.df_norm, **kws)
 
         nx, ny = self.df_norm.T.shape
-        ystart = (ny - 1) % 3
-        npt.assert_array_equal(p.xticks, np.arange(0, nx, 2) + .5)
-        npt.assert_array_equal(p.yticks, np.arange(ystart, ny, 3) + .5)
+        npt.assert_array_equal(p.xticks, np.arange(0, nx, xstep) + .5)
+        npt.assert_array_equal(p.yticks, np.arange(0, ny, ystep) + .5)
         npt.assert_array_equal(p.xticklabels,
-                               self.df_norm.columns[::2])
+                               self.df_norm.columns[0:nx:xstep])
         npt.assert_array_equal(p.yticklabels,
-                               self.df_norm.index[::-1][ystart:ny:3])
+                               self.df_norm.index[0:ny:ystep])
 
     def test_heatmap_annotation(self):
 
         ax = mat.heatmap(self.df_norm, annot=True, fmt=".1f",
                          annot_kws={"fontsize": 14})
-        for val, text in zip(self.x_norm[::-1].flat, ax.texts):
+        for val, text in zip(self.x_norm.flat, ax.texts):
             nt.assert_equal(text.get_text(), "{:.1f}".format(val))
             nt.assert_equal(text.get_fontsize(), 14)
 
@@ -241,8 +319,8 @@ class TestHeatmap(PlotTestCase):
         mask = np.isnan(df.values)
         df_masked = np.ma.masked_where(mask, df)
         ax = mat.heatmap(df, annot=True, fmt='.1f', mask=mask)
-        nt.assert_equal(len(df_masked[::-1].compressed()), len(ax.texts))
-        for val, text in zip(df_masked[::-1].compressed(), ax.texts):
+        nt.assert_equal(len(df_masked.compressed()), len(ax.texts))
+        for val, text in zip(df_masked.compressed(), ax.texts):
             nt.assert_equal("{:.1f}".format(val), text.get_text())
 
     def test_heatmap_annotation_mesh_colors(self):
@@ -259,10 +337,16 @@ class TestHeatmap(PlotTestCase):
         ax = mat.heatmap(self.df_norm, annot=annot_data, fmt=".1f",
                          annot_kws={"fontsize": 14})
 
-        for val, text in zip(annot_data.values[::-1].flat, ax.texts):
+        for val, text in zip(annot_data.values.flat, ax.texts):
             nt.assert_equal(text.get_text(), "{:.1f}".format(val))
             nt.assert_equal(text.get_fontsize(), 14)
 
+    def test_heatmap_annotation_with_limited_ticklabels(self):
+        ax = mat.heatmap(self.df_norm, fmt=".2f", annot=True,
+                         xticklabels=False, yticklabels=False)
+        for val, text in zip(self.x_norm.flat, ax.texts):
+            nt.assert_equal(text.get_text(), "{:.2f}".format(val))
+
     def test_heatmap_cbar(self):
 
         f = plt.figure()
@@ -280,6 +364,8 @@ class TestHeatmap(PlotTestCase):
         nt.assert_equal(len(f.axes), 2)
         plt.close(f)
 
+    @pytest.mark.xfail(mpl.__version__ == "3.1.1",
+                       reason="matplotlib 3.1.1 bug")
     def test_heatmap_axes(self):
 
         ax = mat.heatmap(self.df_norm)
@@ -287,18 +373,18 @@ class TestHeatmap(PlotTestCase):
         xtl = [int(l.get_text()) for l in ax.get_xticklabels()]
         nt.assert_equal(xtl, list(self.df_norm.columns))
         ytl = [l.get_text() for l in ax.get_yticklabels()]
-        nt.assert_equal(ytl, list(self.df_norm.index[::-1]))
+        nt.assert_equal(ytl, list(self.df_norm.index))
 
         nt.assert_equal(ax.get_xlabel(), "")
         nt.assert_equal(ax.get_ylabel(), "letters")
 
         nt.assert_equal(ax.get_xlim(), (0, 8))
-        nt.assert_equal(ax.get_ylim(), (0, 4))
+        nt.assert_equal(ax.get_ylim(), (4, 0))
 
     def test_heatmap_ticklabel_rotation(self):
 
         f, ax = plt.subplots(figsize=(2, 2))
-        mat.heatmap(self.df_norm, ax=ax)
+        mat.heatmap(self.df_norm, xticklabels=1, yticklabels=1, ax=ax)
 
         for t in ax.get_xticklabels():
             nt.assert_equal(t.get_rotation(), 0)
@@ -313,7 +399,7 @@ class TestHeatmap(PlotTestCase):
         df.index = [i * 10 for i in df.index]
 
         f, ax = plt.subplots(figsize=(2, 2))
-        mat.heatmap(df, ax=ax)
+        mat.heatmap(df, xticklabels=1, yticklabels=1, ax=ax)
 
         for t in ax.get_xticklabels():
             nt.assert_equal(t.get_rotation(), 90)
@@ -362,17 +448,14 @@ class TestHeatmap(PlotTestCase):
         npt.assert_array_equal(mask_out, [[True, True], [False, False]])
 
     def test_cbar_ticks(self):
-        max_n_ticks = 3
 
-        locator = mpl.ticker.MaxNLocator(max_n_ticks)
         f, (ax1, ax2) = plt.subplots(2)
         mat.heatmap(self.df_norm, ax=ax1, cbar_ax=ax2,
-                    cbar_kws=dict(ticks=locator))
-        nt.assert_equal(len(ax2.yaxis.get_ticklabels()), max_n_ticks)
-        plt.close(f)
+                    cbar_kws=dict(drawedges=True))
+        assert len(ax2.collections) == 2
 
 
-class TestDendrogram(PlotTestCase):
+class TestDendrogram(object):
     rs = np.random.RandomState(sum(map(ord, "dendrogram")))
 
     x_norm = rs.randn(4, 8) + np.arange(8)
@@ -531,7 +614,7 @@ class TestDendrogram(PlotTestCase):
 
         npt.assert_array_equal(scipy_linkage, linkage)
 
-    @skipif(_no_fastcluster)
+    @pytest.mark.skipif(_no_fastcluster, reason="fastcluster not installed")
     def test_fastcluster_other_method(self):
         import fastcluster
 
@@ -542,7 +625,7 @@ class TestDendrogram(PlotTestCase):
         p = mat._DendrogramPlotter(self.x_norm, **kws)
         npt.assert_array_equal(p.linkage, linkage)
 
-    @skipif(_no_fastcluster)
+    @pytest.mark.skipif(_no_fastcluster, reason="fastcluster not installed")
     def test_fastcluster_non_euclidean(self):
         import fastcluster
 
@@ -568,6 +651,8 @@ class TestDendrogram(PlotTestCase):
         nt.assert_equal(len(ax.collections[0].get_paths()),
                         len(d.dependent_coord))
 
+    @pytest.mark.xfail(mpl.__version__ == "3.1.1",
+                       reason="matplotlib 3.1.1 bug")
     def test_dendrogram_rotate(self):
         kws = self.default_kws.copy()
         kws['rotate'] = True
@@ -613,7 +698,7 @@ class TestDendrogram(PlotTestCase):
         plt.close(f)
 
 
-class TestClustermap(PlotTestCase):
+class TestClustermap(object):
     rs = np.random.RandomState(sum(map(ord, "clustermap")))
 
     x_norm = rs.randn(4, 8) + np.arange(8)
@@ -637,12 +722,15 @@ class TestClustermap(PlotTestCase):
     df_norm_leaves = np.asarray(df_norm.columns[x_norm_leaves])
 
     default_kws = dict(pivot_kws=None, z_score=None, standard_scale=None,
-                       figsize=None, row_colors=None, col_colors=None)
+                       figsize=(10, 10), row_colors=None, col_colors=None,
+                       dendrogram_ratio=.2, colors_ratio=.03,
+                       cbar_pos=(0, .8, .05, .2))
 
     default_plot_kws = dict(metric='euclidean', method='average',
                             colorbar_kws=None,
                             row_cluster=True, col_cluster=True,
-                            row_linkage=None, col_linkage=None)
+                            row_linkage=None, col_linkage=None,
+                            tree_kws=None)
 
     row_colors = color_palette('Set2', df_norm.shape[0])
     col_colors = color_palette('Dark2', df_norm.shape[1])
@@ -761,7 +849,7 @@ class TestClustermap(PlotTestCase):
         kws['z_score'] = True
         kws['standard_scale'] = True
         with nt.assert_raises(ValueError):
-            cm = mat.ClusterGrid(self.df_norm, **kws)
+            mat.ClusterGrid(self.df_norm, **kws)
 
     def test_color_list_to_matrix_and_cmap(self):
         matrix, cmap = mat.ClusterGrid.color_list_to_matrix_and_cmap(
@@ -893,7 +981,7 @@ class TestClustermap(PlotTestCase):
         col_labels = [l.get_text() for l in
                       cm.ax_col_colors.get_yticklabels()]
         nt.assert_equal(cm.col_color_labels, ['col_1', 'col_2'])
-        nt.assert_equal(col_labels[::-1], cm.col_color_labels)
+        nt.assert_equal(col_labels, cm.col_color_labels)
 
     def test_row_col_colors_df_shuffled(self):
         # Tests if colors are properly matched, even if given in wrong order
@@ -908,11 +996,11 @@ class TestClustermap(PlotTestCase):
 
         row_colors = pd.DataFrame({'row_annot': list(self.row_colors)},
                                   index=self.df_norm.index)
-        kws['row_colors'] = row_colors.ix[shuffled_inds]
+        kws['row_colors'] = row_colors.loc[shuffled_inds]
 
         col_colors = pd.DataFrame({'col_annot': list(self.col_colors)},
                                   index=self.df_norm.columns)
-        kws['col_colors'] = col_colors.ix[shuffled_cols]
+        kws['col_colors'] = col_colors.loc[shuffled_cols]
 
         cm = mat.clustermap(self.df_norm, **kws)
         nt.assert_equal(list(cm.col_colors)[0], list(self.col_colors))
@@ -962,7 +1050,7 @@ class TestClustermap(PlotTestCase):
         col_labels = [l.get_text() for l in
                       cm2.ax_col_colors.get_yticklabels()]
         nt.assert_equal(cm2.col_color_labels, ['col_1', 'col_2'])
-        nt.assert_equal(col_labels[::-1], cm2.col_color_labels)
+        nt.assert_equal(col_labels, cm2.col_color_labels)
 
     def test_row_col_colors_series(self):
         kws = self.default_kws.copy()
@@ -996,11 +1084,11 @@ class TestClustermap(PlotTestCase):
 
         row_colors = pd.Series(list(self.row_colors), name='row_annot',
                                index=self.df_norm.index)
-        kws['row_colors'] = row_colors.ix[shuffled_inds]
+        kws['row_colors'] = row_colors.loc[shuffled_inds]
 
         col_colors = pd.Series(list(self.col_colors), name='col_annot',
                                index=self.df_norm.columns)
-        kws['col_colors'] = col_colors.ix[shuffled_cols]
+        kws['col_colors'] = col_colors.loc[shuffled_cols]
 
         cm = mat.clustermap(self.df_norm, **kws)
 
@@ -1023,6 +1111,25 @@ class TestClustermap(PlotTestCase):
         nt.assert_equal(list(cm.row_colors),
                         [(1.0, 1.0, 1.0)] + list(self.row_colors[1:]))
 
+    def test_row_col_colors_ignore_heatmap_kwargs(self):
+
+        g = mat.clustermap(self.rs.uniform(0, 200, self.df_norm.shape),
+                           row_colors=self.row_colors,
+                           col_colors=self.col_colors,
+                           cmap="Spectral",
+                           norm=mpl.colors.LogNorm(),
+                           vmax=100)
+
+        assert np.array_equal(
+            np.array(self.row_colors)[g.dendrogram_row.reordered_ind],
+            g.ax_row_colors.collections[0].get_facecolors()[:, :3]
+        )
+
+        assert np.array_equal(
+            np.array(self.col_colors)[g.dendrogram_col.reordered_ind],
+            g.ax_col_colors.collections[0].get_facecolors()[:, :3]
+        )
+
     def test_mask_reorganization(self):
 
         kws = self.default_kws.copy()
@@ -1044,7 +1151,7 @@ class TestClustermap(PlotTestCase):
         kws = self.default_kws.copy()
         xtl = np.arange(self.df_norm.shape[1])
         kws["xticklabels"] = list(xtl)
-        ytl = self.letters.ix[:self.df_norm.shape[0]]
+        ytl = self.letters.loc[:self.df_norm.shape[0]]
         kws["yticklabels"] = ytl
 
         g = mat.clustermap(self.df_norm, **kws)
@@ -1053,7 +1160,126 @@ class TestClustermap(PlotTestCase):
         ytl_actual = [t.get_text() for t in g.ax_heatmap.get_yticklabels()]
 
         xtl_want = xtl[g.dendrogram_col.reordered_ind].astype("<U1")
-        ytl_want = ytl[g.dendrogram_row.reordered_ind].astype("<U1")[::-1]
+        ytl_want = ytl[g.dendrogram_row.reordered_ind].astype("<U1")
 
         npt.assert_array_equal(xtl_actual, xtl_want)
         npt.assert_array_equal(ytl_actual, ytl_want)
+
+    def test_noticklabels(self):
+
+        kws = self.default_kws.copy()
+        kws["xticklabels"] = False
+        kws["yticklabels"] = False
+
+        g = mat.clustermap(self.df_norm, **kws)
+
+        xtl_actual = [t.get_text() for t in g.ax_heatmap.get_xticklabels()]
+        ytl_actual = [t.get_text() for t in g.ax_heatmap.get_yticklabels()]
+        nt.assert_equal(xtl_actual, [])
+        nt.assert_equal(ytl_actual, [])
+
+    def test_size_ratios(self):
+
+        # The way that wspace/hspace work in GridSpec, the mapping from input
+        # ratio to actual width/height of each axes is complicated, so this
+        # test is just going to assert comparative relationships
+
+        kws1 = self.default_kws.copy()
+        kws1.update(dendrogram_ratio=.2, colors_ratio=.03,
+                    col_colors=self.col_colors, row_colors=self.row_colors)
+
+        kws2 = kws1.copy()
+        kws2.update(dendrogram_ratio=.3, colors_ratio=.05)
+
+        g1 = mat.clustermap(self.df_norm, **kws1)
+        g2 = mat.clustermap(self.df_norm, **kws2)
+
+        assert (g2.ax_col_dendrogram.get_position().height
+                > g1.ax_col_dendrogram.get_position().height)
+
+        assert (g2.ax_col_colors.get_position().height
+                > g1.ax_col_colors.get_position().height)
+
+        assert (g2.ax_heatmap.get_position().height
+                < g1.ax_heatmap.get_position().height)
+
+        assert (g2.ax_row_dendrogram.get_position().width
+                > g1.ax_row_dendrogram.get_position().width)
+
+        assert (g2.ax_row_colors.get_position().width
+                > g1.ax_row_colors.get_position().width)
+
+        assert (g2.ax_heatmap.get_position().width
+                < g1.ax_heatmap.get_position().width)
+
+        kws1 = self.default_kws.copy()
+        kws1.update(col_colors=self.col_colors)
+        kws2 = kws1.copy()
+        kws2.update(col_colors=[self.col_colors, self.col_colors])
+
+        g1 = mat.clustermap(self.df_norm, **kws1)
+        g2 = mat.clustermap(self.df_norm, **kws2)
+
+        assert (g2.ax_col_colors.get_position().height
+                > g1.ax_col_colors.get_position().height)
+
+        kws1 = self.default_kws.copy()
+        kws1.update(dendrogram_ratio=(.2, .2))
+
+        kws2 = kws1.copy()
+        kws2.update(dendrogram_ratio=(.2, .3))
+
+        g1 = mat.clustermap(self.df_norm, **kws1)
+        g2 = mat.clustermap(self.df_norm, **kws2)
+
+        assert (g2.ax_row_dendrogram.get_position().width
+                == g1.ax_row_dendrogram.get_position().width)
+
+        assert (g2.ax_col_dendrogram.get_position().height
+                > g1.ax_col_dendrogram.get_position().height)
+
+    def test_cbar_pos(self):
+
+        kws = self.default_kws.copy()
+        kws["cbar_pos"] = (.2, .1, .4, .3)
+
+        g = mat.clustermap(self.df_norm, **kws)
+        pos = g.ax_cbar.get_position()
+        assert pytest.approx(tuple(pos.p0)) == kws["cbar_pos"][:2]
+        assert pytest.approx(pos.width) == kws["cbar_pos"][2]
+        assert pytest.approx(pos.height) == kws["cbar_pos"][3]
+
+        kws["cbar_pos"] = None
+        g = mat.clustermap(self.df_norm, **kws)
+        assert g.ax_cbar is None
+
+    def test_square_warning(self):
+
+        kws = self.default_kws.copy()
+        g1 = mat.clustermap(self.df_norm, **kws)
+
+        with pytest.warns(UserWarning):
+            kws["square"] = True
+            g2 = mat.clustermap(self.df_norm, **kws)
+
+        g1_shape = g1.ax_heatmap.get_position().get_points()
+        g2_shape = g2.ax_heatmap.get_position().get_points()
+        assert np.array_equal(g1_shape, g2_shape)
+
+    def test_clustermap_annotation(self):
+
+        g = mat.clustermap(self.df_norm, annot=True, fmt=".1f")
+        for val, text in zip(np.asarray(g.data2d).flat, g.ax_heatmap.texts):
+            assert text.get_text() == "{:.1f}".format(val)
+
+        g = mat.clustermap(self.df_norm, annot=self.df_norm, fmt=".1f")
+        for val, text in zip(np.asarray(g.data2d).flat, g.ax_heatmap.texts):
+            assert text.get_text() == "{:.1f}".format(val)
+
+    def test_tree_kws(self):
+
+        rgb = (1, .5, .2)
+        g = mat.clustermap(self.df_norm, tree_kws=dict(color=rgb))
+        for ax in [g.ax_col_dendrogram, g.ax_row_dendrogram]:
+            tree, = ax.collections
+            assert tuple(tree.get_color().squeeze())[:3] == rgb
