@@ -1,13 +1,11 @@
 import numpy as np
 import matplotlib as mpl
-from distutils.version import LooseVersion
 import nose
 import matplotlib.pyplot as plt
 import nose.tools as nt
 import numpy.testing as npt
 
-from . import PlotTestCase
-from .. import rcmod
+from .. import rcmod, palettes, utils
 
 
 class RCParamTester(object):
@@ -88,12 +86,30 @@ class TestAxesStyle(RCParamTester):
         nt.assert_equal(mpl.rcParams["lines.linewidth"], 4)
         rcmod.set()
 
-    def test_reset_defaults(self):
+    def test_set_with_palette(self):
 
-        # Changes to the rc parameters make this test hard to manage
-        # on older versions of matplotlib, so we'll skip it
-        if LooseVersion(mpl.__version__) < LooseVersion("1.3"):
-            raise nose.SkipTest
+        rcmod.reset_orig()
+
+        rcmod.set(palette="deep")
+        assert utils.get_color_cycle() == palettes.color_palette("deep", 10)
+        rcmod.reset_orig()
+
+        rcmod.set(palette="deep", color_codes=False)
+        assert utils.get_color_cycle() == palettes.color_palette("deep", 10)
+        rcmod.reset_orig()
+
+        pal = palettes.color_palette("deep")
+        rcmod.set(palette=pal)
+        assert utils.get_color_cycle() == palettes.color_palette("deep", 10)
+        rcmod.reset_orig()
+
+        rcmod.set(palette=pal, color_codes=False)
+        assert utils.get_color_cycle() == palettes.color_palette("deep", 10)
+        rcmod.reset_orig()
+
+        rcmod.set()
+
+    def test_reset_defaults(self):
 
         rcmod.reset_defaults()
         self.assert_rc_params(mpl.rcParamsDefault)
@@ -101,11 +117,6 @@ class TestAxesStyle(RCParamTester):
 
     def test_reset_orig(self):
 
-        # Changes to the rc parameters make this test hard to manage
-        # on older versions of matplotlib, so we'll skip it
-        if LooseVersion(mpl.__version__) < LooseVersion("1.3"):
-            raise nose.SkipTest
-
         rcmod.reset_orig()
         self.assert_rc_params(mpl.rcParamsOrig)
         rcmod.set()
@@ -176,7 +187,24 @@ class TestPlottingContext(RCParamTester):
         self.assert_rc_params(orig_params)
 
 
-class TestFonts(PlotTestCase):
+class TestPalette(object):
+
+    def test_set_palette(self):
+
+        rcmod.set_palette("deep")
+        assert utils.get_color_cycle() == palettes.color_palette("deep", 10)
+
+        rcmod.set_palette("pastel6")
+        assert utils.get_color_cycle() == palettes.color_palette("pastel6", 6)
+
+        rcmod.set_palette("dark", 4)
+        assert utils.get_color_cycle() == palettes.color_palette("dark", 4)
+
+        rcmod.set_palette("Set2", color_codes=True)
+        assert utils.get_color_cycle() == palettes.color_palette("Set2", 8)
+
+
+class TestFonts(object):
 
     def test_set_font(self):
 
@@ -210,12 +238,8 @@ class TestFonts(PlotTestCase):
 
     def test_different_sans_serif(self):
 
-        if LooseVersion(mpl.__version__) < LooseVersion("1.4"):
-            raise nose.SkipTest
-
         rcmod.set()
-        rcmod.set_style(rc={"font.sans-serif":
-                            ["Verdana"]})
+        rcmod.set_style(rc={"font.sans-serif": ["Verdana"]})
 
         _, ax = plt.subplots()
         ax.set_xlabel("foo")
@@ -240,7 +264,7 @@ def has_verdana():
     import matplotlib.font_manager as mplfm
     try:
         verdana_font = mplfm.findfont('Verdana', fallback_to_default=False)
-    except:
+    except:  # noqa
         # if https://github.com/matplotlib/matplotlib/pull/3435
         # gets accepted
         return False
@@ -248,7 +272,7 @@ def has_verdana():
     try:
         unlikely_font = mplfm.findfont("very_unlikely_to_exist1234",
                                        fallback_to_default=False)
-    except:
+    except:  # noqa
         # if matched verdana but not unlikely, Verdana must exist
         return True
     # otherwise -- if they match, must be the same default
